unit direct3d;
// Direct3D Immediate mode test by Massimo Bardetti - bigben@dircon.co.uk.
// Delphi Wrapper for Direct3d Device

interface
{$A+} {$Z4}
uses Forms, SysUtils,  OLE2, Windows , DDraw, D3D, D3DCaps, D3DTYPES, DirectDraw;

type
  ENUMDEVINFO = record
  cm : D3DCOLORMODEL;
  pHWGuid : PGUID ;
  HWDesc : D3DDEVICEDESC;
  pSWGuid : PGUID;
  SWDesc : D3DDEVICEDESC;
end;

type
T3dTextureFormat = record
     ddsd : DDSURFACEDESC;
     bPalettized : Boolean;
     redBPP : integer;
     greenBPP : integer;
     blueBPP : integer;
     indexBPP : integer;
end;

type
  TD3D = class ( TObject )
  private
    FZBuffer : IDirectDrawSurface;
    scrScreen : TDDSCreen;
    CurrTextureFormat : integer;
    function EnumTextureFormats : boolean;
    function CreateZBuffer(bpp : integer; dwMemFlags : DWORD) : boolean;
  public
    D3D : IDirect3D ;
    D3DDevice : IDirect3dDevice;
    D3DViewPort : IDirect3DViewport;
    ThisTextureFormat : T3dTextureFormat;

    constructor Create ( pScreen : TDDScreen ) ;
    destructor Destroy ; override ;
    function SetMode (cm : D3DCOLORMODEL) : boolean;
    function CreateViewPort : boolean;
    function GetTextureNum : integer;
    function Gettexture(n : integer; var tf : T3DTextureFormat) : boolean;
  end ;

function EnumDevCallBack ( lpGuid: PGUID ;
      lpDeviceDescription: LPSTR ; lpDeviceName: LPSTR ;
      const lpD3DHWDeviceDesc: D3DDEVICEDESC ;
      const lpD3DHELDeviceDesc: D3DDEVICEDESC ;
      lpUserArg: Pointer ): HRESULT ; stdcall ;

function EnumTextureFormatsCallback( const lpDdsd: DDSURFACEDESC ;
      lpUserArg: Pointer ): HRESULT ; stdcall ;


var
   DDResult : HRESULT;
   TextureFormat : array [0..50] of T3dTextureFormat;
   NumtextureFormats : Integer;

implementation


constructor TD3D.Create (pScreen : TDDScreen) ;
begin
 // D3d := nil;
  pScreen.DDraw.QueryInterface(IID_IDirect3D, D3D);
  if (D3d = nil) then
     raise Exception.Create('No 3d surface!!');
  scrScreen := pScreen;
end;

destructor TD3D.Destroy ;
begin

     if FZBuffer <> nil then
        FZBuffer.release;

     if (D3DViewPort <> nil) then
        D3DViewport.Release;

     if (D3dDevice <> nil) then
        D3DDevice.Release;

     if (D3D <> nil) then
        D3D.release;

end;


function TD3D.CreateZBuffer(bpp : integer; dwMemFlags : DWORD) : boolean;
var
   pBB : IDirectDrawSurface;
   ds : DDSURFACEDESC;
   cx,cy : integer;
begin
     pBB := scrScreen.BackSurface;
     pBB.GetSurfaceDesc(ds);

     cx := scrScreen.width; //ds.dwWidth;
     cy := scrScreen.height; //ds.dwHeight;

     ZeroMemory(@ds,sizeof(ds));
     ds.dwSize := sizeof(ds);
     ds.dwFlags := DDSD_WIDTH + DDSD_HEIGHT +DDSD_CAPS + DDSD_ZBUFFERBITDEPTH; //
     ds.dwHeight := cy;
     ds.dwWidth := cx;
     ds.ddsCaps.dwCaps := DDSCAPS_ZBUFFER + dwMemFlags;
     ds.DWZBufferBitDepth := bpp;

     DDResult := scrScreen.DDraw.CreateSurface(ds, FZBuffer, nil);
     if DDResult <> DD_OK then
     raise Exception.Create ( Format (
        'IDirectDraw.CreateSurface Z-Buffer failed: %x', [ DDResult ] ) ) ;

     pBB.AddAttachedSurface(FZBuffer);

     Result := True;
end;

function TD3D.SetMode (cm : D3DCOLORMODEL) : Boolean;
var
   edi : ENUMDEVINFO;
   lpGuid : PGUID;
   b : boolean;
   hw, drv : DDCAPS;
   bpp : integer;
begin
     edi.cm := cm;
     edi.pHWGuid := nil;
     edi.pSWGuid := nil;
     D3D.EnumDevices(EnumDevCallBack, @edi);
     if (edi.pHWGuid = nil) and (edi.pSWGuid = nil) then
       begin
        Result := False;
        exit;
       end;


     lpGuid := nil;
     if (edi.pHWGuid = nil) then
        begin
             lpGuid := edi.pSWGuid;
             b := CreateZBuffer(16, DDSCAPS_SYSTEMMEMORY);
        end
     else
         begin
              lpGuid := edi.pHWGuid;

              scrScreen.DDraw.GetCaps(hw,drv);

              bpp := 16;
              if (hw.dwZBufferBitDepths and DDBD_32) <> 0 then
                 bpp := 32
              else if (hw.dwZBufferBitDepths and DDBD_24) <> 0 then
                   bpp := 24
              else if (hw.dwZBufferBitDepths and DDBD_16) <> 0 then
                   bpp := 16
              else if (hw.dwZBufferBitDepths and DDBD_8) <> 0 then
                   bpp := 8
              else if (hw.dwZBufferBitDepths and DDBD_4) <> 0 then
                   bpp := 4
              else if (hw.dwZBufferBitDepths and DDBD_2) <> 0then
                   bpp := 2
              else if (hw.dwZBufferBitDepths and DDBD_1) <> 0then
                   bpp := 1;

              b := CreateZBuffer(bpp, DDSCAPS_VIDEOMEMORY);
         end;

     if not b then
        begin
             Result := False;
             exit;
        end;

     DDResult := scrScreen.BackSurface.QueryInterface(lpGuid^, D3dDevice);
     if DdResult <> DD_OK then
        Result := False;

     EnumTextureFormats;
end;

function TD3D.CreateViewPort : boolean;
begin
     DDResult := D3D.CreateViewPort(D3DViewPort, nil);
     if DDResult <> DD_OK then
      begin
        Result := False;
        exit;
      end;

     DDResult := D3DDevice.AddViewPort(D3DViewPort);
     if DDResult <> DD_OK then
      begin
        Result := False;
        exit;
      end;

     Result := True;
end;

function EnumDevCallBack ( lpGuid: PGUID ;
      lpDeviceDescription: LPSTR ; lpDeviceName: LPSTR ;
      const lpD3DHWDeviceDesc: D3DDEVICEDESC ;
      const lpD3DHELDeviceDesc: D3DDEVICEDESC ;
      lpUserArg: Pointer ): HRESULT ; stdcall ;
var
   pdi : pointer;
begin
     pdi := lpUserArg;

     if lpD3dHWDeviceDesc.dcmColorModel = ENUMDEVINFO (pdi^).cm then
      begin
        ENUMDEVINFO (pdi^).HWDesc := lpD3DHWDeviceDesc;
        ENUMDEVINFO (pdi^).pHWGuid := lpGuid;
      end;

     if lpD3dHELDeviceDesc.dcmColorModel = ENUMDEVINFO (pdi^).cm then
      begin
           ENUMDEVINFO (pdi^).SWDesc := LpD3DHELDeviceDesc;
           ENUMDEVINFO (pdi^).pSWGuid := lpGuid;
      end;

     Result := D3DENUMRET_OK;
end;


function EnumTextureFormatsCallback( const lpDdsd: DDSURFACEDESC ;
      lpUserArg: Pointer ): HRESULT ; stdcall ;
var
   m : cardinal;
   r,g,b : integer;
   pStartFormat : ^integer;
begin
    pStartFormat := lpUserArg;
//    /*
//     * Record the DDSURFACEDESC of this texture format
//     */
    ZeroMemory(@TextureFormat[NumTextureFormats],sizeof(T3DTextureFormat));
    CopyMemory(@TextureFormat[NumTextureFormats].ddsd, @lpDDSD, sizeof(DDSURFACEDESC));
//    /*
//     * Is this format palettized?  How many bits?  Otherwise, how many RGB
//     * bits?
//     */
    if (lpDDSD.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED8) <> 0 then
      begin
        TextureFormat[NumTextureFormats].bPalettized := TRUE;
        TextureFormat[NumTextureFormats].IndexBPP := 8;
      end
    else if (lpDDSD.ddpfPixelFormat.dwFlags and DDPF_PALETTEINDEXED4) <> 0 then
      begin
        TextureFormat[NumTextureFormats].bPalettized := TRUE;
        TextureFormat[NumTextureFormats].IndexBPP := 4;
      end
    else
      begin
        TextureFormat[NumTextureFormats].bPalettized := FALSE;
        TextureFormat[NumTextureFormats].IndexBPP := 0;
        m := lpDDSD.ddpfPixelFormat.dwRBitMask;
        r := 0;
        while (m and 1) = 0 do
              begin
                   Inc(r);
                   m := m shr 1;
              end;

        r:=0;
        while (m and 1) <> 0 do
              begin
                   Inc(r);
                   m := m shr 1;
              end;

        m := lpDDSD.ddpfPixelFormat.dwRBitMask;
        g := 0;
        while (m and 1) = 0 do
              begin
                   Inc(g);
                   m := m shr 1;
              end;

        g:=0;
        while (m and 1) <> 0 do
              begin
                   Inc(g);
                   m := m shr 1;
              end;

        m := lpDDSD.ddpfPixelFormat.dwRBitMask;
        b := 0;
        while (m and 1) = 0 do
              begin
                   Inc(b);
                   m := m shr 1;
              end;

        b:=0;
        while (m and 1) <> 0 do
              begin
                   Inc(b);
                   m := m shr 1;
              end;

        TextureFormat[NumTextureFormats].RedBPP := r;
        TextureFormat[NumTextureFormats].GreenBPP := g;
        TextureFormat[NumTextureFormats].BlueBPP := b;
      end;
//    /*
//     * If lpStarFormat is -1, this is the first format.  Select it.
//     */
    if (pStartFormat^ = -1) then
        pStartFormat^ := NumTextureFormats;
//    /*
//     * If this format is paletted, select it.
//     */
    if (TextureFormat[NumTextureFormats].bPalettized) then
        pStartFormat^ := NumTextureFormats;

    inc(NumTextureFormats);
    Result := DDENUMRET_OK;
end;

{/*
 * D3DAppIEnumTextureFormats
 * Get a list of available texture map formats from the Direct3D driver by
 * enumeration.  Choose a default format (paletted is prefered).
 */}
function TD3D.EnumTextureFormats : boolean;
var
   StartFormat : integer;
begin
//    /*
//     * Set the default format to -1 to let the callback know it's being
//     * called for the first time.
//     */
    StartFormat := -1;
    NumTextureFormats := 0;
    DDResult := D3DDevice.EnumTextureFormats(EnumTextureFormatsCallback,@StartFormat);
    if (DDResult <> DD_OK) then
       raise Exception.Create('Cannot enum textures');

    CopyMemory(@ThisTextureFormat, @TextureFormat[StartFormat],sizeof(T3DTextureFormat));
    CurrTextureFormat := StartFormat;
    Result := TRUE;
end;


function TD3D.GetTextureNum : integer;
begin
     result := NumtextureFormats;
end;

function TD3D.GetTexture(n : integer; var tf : T3DTextureFormat) : boolean;
begin
     if n > numtextureFormats - 1 then
        begin
             Result := false;
             exit;
        end;

     tf := TextureFormat[n];
     result := True;
end;



end.
